using System;
using System.Collections.Generic;
using Produmex.Foundation.Collections;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DataSetClient;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.Utilities;
using Produmex.Sbo.Logex.WorkFlowSteps;
using Produmex.Foundation.Messages;

namespace WorkflowScript_ComponentWeighingProductionScriptSFF
{
    /// <summary>
    /// 
    /// </summary>
    [FlowCustomization(nameof(Workflows.ComponentWeighingProductionScript))]
    public class WorkflowScript_ComponentWeighingProductionScriptSFF : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);


        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        ISboProviderService m_sboProviderService;

        // Output parameters
        // <none>

        // Sub-flows
        WorkflowInstanceBase m_timeRegistrationSubFlow;
        protected bool m_subFlowBackRequested;



        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_ComponentWeighingProductionScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_ComponentWeighingProductionScriptSFF(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
            m_timeRegistrationSubFlow = Create("TimeRegistrationScript");
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            //// Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            StepHelper stepHelper = new StepHelper
            {
                DefaultCultureInfo = this.DefaultCultureInfo,
                ShowCustomizedScreen = session.ShowCustomizedScreen,
                WorkflowId = this.WorkflowId,
                WaitForMessage = this.WaitForMessage,
                SboProviderService = m_sboProviderService,
                DeviceId = session.DeviceInstanceId
            };

            ////reset in case of error

            string deviceID = session.DeviceInstanceId;
            string query = null;

            Produmex.Foundation.Messages.Message msg = null;

            string productionLineCode = null;
            string productionLineCodeForDevice = null;
            DataSet dsProductionLines = null;
            bool goBack = false;
            bool screenRefresh = false;
            string initialErrorKey = null;
            string pickToLocationCode = null;
            string weighingInputLocationCode = null;
            string weighingOutputLocationCode = null;
            string restLocationCode = null;
            string outputLocationCode = null;
            int? productionOrderDocEntry = null;
            int? productionOrderLineNum = null;
            DataSet dsProductionOrder = null;
            string returnAfterTimeRegistration = null;
            double? nrOfProdBatches = null;
            string printLabelsOnPartialWeighingOption = null;
            string buttonLocalizationForPartialWeighingButton = null;

            bool lockPickedItemsForPickTo = false;

            PmxOseWarehouseInfo whsInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), deviceID);


            m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
              delegate (PmxDbConnection conn, object[] parameters)
              {
                  PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
                  PmxDictionary<string, PmxParameterValue> prodParams = prdProv.GetParametersProductionController();

                  printLabelsOnPartialWeighingOption = PmxParameterValueProvider.GetParameterValue<string>(prodParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_PRINT_LABELS_FOR_OPEN_PARTIAL_WEIGHING_OPTION, PmxYesNoAskOptionCodes.NO);

                  return null;
              });

            switch (printLabelsOnPartialWeighingOption)
            {
                case PmxYesNoAskOptionCodes.YES:
                    buttonLocalizationForPartialWeighingButton = "MSG_BUTTON_PARTIAL_WEIGHING_YES";
                    break;
                case PmxYesNoAskOptionCodes.ASK:
                    buttonLocalizationForPartialWeighingButton = "MSG_BUTTON_PARTIAL_WEIGHING_ASK";
                    break;
                default:
                    buttonLocalizationForPartialWeighingButton = "MSG_BUTTON_PARTIAL_WEIGHING";
                    break;
            }

        Step_SelectProductionLine:

            #region SELECT PRODUCTION LINE

            /* BOYUM ROPI 001 START */

            string productionLineZone = null;

            query = @"SELECT 
	                    DISTINCT ""PMX_OSEL"".""ParentCode"" AS ""Code"", ""PMX_OSEL"".""ParentCode"" AS ""Description"", '' AS ""WarehouseCode"", '' as ""InputLocationCode"", '' as ""PickToLocationCode"", '' as ""LockPickedItemsForPickTo"", 
	                    '' as ""OutputLocationCode"", '' as ""RestLocationCode"", '' AS ""WarehouseName"" 
                    FROM 
	                    ""PMX_OSSL"" 
	                    INNER JOIN ""PMX_OSEL"" ON ""PMX_OSEL"".""Code"" = ""PMX_OSSL"".""Code"" 
	                    INNER JOIN ""PMX_OSWH"" ON ""PMX_OSEL"".""PmxWhsCode"" = ""PMX_OSWH"".""Code"" 
	                    INNER JOIN ""OWHS"" ON ""OWHS"".""WhsCode"" = ""PMX_OSWH"".""SboWhsCode"" AND ""OWHS"".""U_PMX_IMBP"" = 'Y' 
	                    INNER JOIN ""PMX_OSPL"" ON ""PMX_OSEL"".""Code"" = ""PMX_OSPL"".""Code"" 
                    WHERE 
	                    ""PMX_OSSL"".""IsActive"" = 'Y' 
                    ORDER BY ""PMX_OSEL"".""ParentCode""";

            dsProductionLines = m_sboProviderService.RunView(false, null, null, query);

            session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
               DefaultCultureInfo.Get(), BuildParamCollection(
               "InitialErrorKey", initialErrorKey,
               "TitleKey", "Select Zone",
               "LocationDS", dsProductionLines),
               WorkflowId,
               nameof(ComponentWeighingProductionScript_Screens.SelectLocationScreen1));
            msg = WaitForMessage();

            if (msg.Name.EndsWith(".LocationSelected"))
            {
                productionLineZone = ExtractParameter<string>(msg.Parameters, "locationCode");
            }

            query = BuildQuery.GetProductionLines(whsInfo.PmxWarehouseCode, deviceID, productionLineZone);
            /* BOYUM ROPI 001 END */

            // query = BuildQuery.GetProductionLines(whsInfo.PmxWarehouseCode, deviceID);
            dsProductionLines = m_sboProviderService.RunView(false, null, null, query);

            if (!goBack)
            {
                if (this.GetNumberOfRows(dsProductionLines) > 0  // Why this check? why not
                    && !string.IsNullOrEmpty(productionLineCodeForDevice))
                {
                    productionLineCode = productionLineCodeForDevice;
                    goto Step_GetProductionLineInfo;
                }

                if (this.GetNumberOfRows(dsProductionLines) == 1)
                {
                    //auto select location
                    productionLineCode = Convert.ToString(dsProductionLines.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name]);
                    goto Step_GetProductionLineInfo;
                }
            }
            goBack = false;

            session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
               DefaultCultureInfo.Get(), BuildParamCollection(
               "InitialErrorKey", initialErrorKey,
               "TitleKey", "MSG_TITLE_SELECT_PRODUCTION_LINE",
               "LocationDS", dsProductionLines),
               WorkflowId,
               nameof(ComponentWeighingProductionScript_Screens.SelectLocationScreen1));
            msg = WaitForMessage();
            goBack = false;
            initialErrorKey = null;

            if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_SelectProductionLine;
            }
            if (msg.Name.EndsWith(".LocationSelected"))
            {
                productionLineCode = ExtractParameter<string>(msg.Parameters, "locationCode");
                goto Step_GetProductionLineInfo;
            }

            //back requested
            return;

        #endregion


        Step_GetProductionLineInfo:

            #region GET PRODUCTION LINE INFO


            if (this.GetNumberOfRows(dsProductionLines) > 0)
            {
                DataRow[] rows = dsProductionLines.Tables[0].Select(LocationDataSet.Columns.CODE.Name + " = '" + productionLineCode + "'");
                if (rows.Length > 0)
                {
                    DataRow row = rows[0];
                    pickToLocationCode = row[PmxOseProductionLineTable.Columns.PickToLocationCode.NAME] as string;
                    weighingOutputLocationCode = row[PmxOseProductionLineTable.Columns.InputLocationCode.NAME] as string;
                    weighingInputLocationCode = pickToLocationCode;
                    lockPickedItemsForPickTo = (row[PmxOseProductionLineTable.Columns.LockPickedItemsForPickTo.NAME] as string) == SboBooleanColumnValues.Y.VALUE && !string.IsNullOrEmpty(pickToLocationCode);

                    if (string.IsNullOrEmpty(pickToLocationCode))
                    {
                        //Set input location as weighing output location to
                        weighingInputLocationCode = weighingOutputLocationCode;
                    }



                    outputLocationCode = row[PmxOseProductionLineTable.Columns.OutputLocationCode.NAME] as string;
                    restLocationCode = row[PmxOseProductionLineTable.Columns.RestLocationCode.NAME] as string;
                    string warehouseCode = row[LocationDataSet.Columns.WAREHOUSECODE.Name] as string;
                    if (warehouseCode != whsInfo.PmxWarehouseCode)
                    {
                        whsInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), warehouseCode);
                    }

                }
                else
                {
                    //production line not found!
                    //This should not happen
                    goBack = true;
                    goto Step_SelectProductionLine;
                }
            }
            else
            {
                //production line not found!
                //This should not happen
                goBack = true;
                goto Step_SelectProductionLine;
            }

            goto Step_SelectProductionOrder;

        #endregion

        Step_SelectProductionOrder:

            #region SELECT PRODUCTION ORDER

            /* BOYUM ROPI 003 START */

            string AllergenesType = null;
            DataSet dsAllergenes = null;

            query = @"
					SELECT 
	                    'WA' AS ""Code"", 'With Allergens' AS ""Description"", '' AS ""WarehouseCode"", '' as ""InputLocationCode"", '' as ""PickToLocationCode"", '' as ""LockPickedItemsForPickTo"", '' as ""OutputLocationCode"", '' as ""RestLocationCode"", '' AS ""WarehouseName"" 
					UNION
					SELECT 
	                    'WOA' AS ""Code"", 'Without Allergens' AS ""Description"", '' AS ""WarehouseCode"", '' as ""InputLocationCode"", '' as ""PickToLocationCode"", '' as ""LockPickedItemsForPickTo"", '' as ""OutputLocationCode"", '' as ""RestLocationCode"", '' AS ""WarehouseName"" 
            ";

            dsAllergenes = m_sboProviderService.RunView(false, null, null, query);

            session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
               DefaultCultureInfo.Get(), BuildParamCollection(
               "InitialErrorKey", initialErrorKey,
               "TitleKey", " ",
               "LocationDS", dsAllergenes),
               WorkflowId,
               nameof(ComponentWeighingProductionScript_Screens.SelectLocationScreen1));
            msg = WaitForMessage();

            if (msg.Name.EndsWith(".LocationSelected"))
            {
                AllergenesType = ExtractParameter<string>(msg.Parameters, "locationCode");
                s_log.Error("ROPIDEV AllergenesType: " + AllergenesType);
            }

            /*  get PO based AllergenesType */

            /* BOYUM ROPI 003 END */



            productionOrderDocEntry = null;

            query = BuildQuery.GetProductionOrders(productionLineCode, productionOrderDocEntry, m_sboProviderService.GetDbTool());

            /* BOYUM ROPI 004 END */
            if (AllergenesType == "WOA")
            {
                query = query.Substring(0, query.IndexOf("ORDER BY")) + @"  AND ""OWOR"".""DocEntry""  in ( SELECT ""WOR1"".""DocEntry"" FROM ""WOR1"" left join ""PMX_ITAL"" on ""PMX_ITAL"".""ItemCode"" = ""WOR1"".""ItemCode"" GROUP BY ""WOR1"".""DocEntry"" HAVING SUM(CASE WHEN ""PMX_ITAL"".""InternalKey"" is null THEN 0 ELSE 1 END)  = 0 )";
            }

            if (AllergenesType == "WA")
            {
                query = query.Substring(0, query.IndexOf("ORDER BY")) + @"  AND ""OWOR"".""DocEntry""  in ( SELECT ""WOR1"".""DocEntry"" FROM ""WOR1"" left join ""PMX_ITAL"" on ""PMX_ITAL"".""ItemCode"" = ""WOR1"".""ItemCode"" GROUP BY ""WOR1"".""DocEntry"" HAVING SUM(CASE WHEN ""PMX_ITAL"".""InternalKey"" is null THEN 0 ELSE 1 END)  > 0 )";
            }

            s_log.Error("ROPIDEV query: " + query);
            /* BOYUM ROPI 004 END */

            dsProductionOrder = m_sboProviderService.RunView(false, null, null, query);

            if (!goBack)
            {
                //if ( this.GetNumberOfRows( dsProductionOrder ) == 1  // Why is this?
                //    && productionOrderDocEntry.HasValue )
                //{
                //    goto Step_SelectProductionLine;
                //}
            }

            // Show screen with list
            session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectProductionOrderScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_PRODUCTION_ORDER",
                    "ProductionOrderDS", dsProductionOrder,
                    "EnabledToolBarButtons", ToolBarButtons.Back | ToolBarButtons.Forward | ToolBarButtons.RefreshContent | ToolBarButtons.Keyboard | ToolBarButtons.Time),
                WorkflowId,
                nameof(ComponentWeighingProductionScript_Screens.SelectProductionOrderScreen1));
            msg = WaitForMessage();
            goBack = false;
            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;
                goto Step_SelectProductionLine;
            }
            if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_SelectProductionOrder;
            }
            if (msg.Name.EndsWith(".TimeRequested"))
            {
                productionOrderDocEntry = ExtractParameter<int>(msg.Parameters, "productionOrderDocEntry");
                goto Step_TimeRegistration;
            }
            if (!msg.Name.EndsWith(".ProductionOrderSelected"))
            {
                initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                goto Step_SelectProductionOrder;
            }

            //User selected a production order
            productionOrderDocEntry = ExtractParameter<int>(msg.Parameters, "productionOrderDocEntry");
            DataRow[] selectedProdOrderRow = dsProductionOrder.Tables[0].Select(DataSetCommandHelper.EscapeColumnName(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name) + " = " + productionOrderDocEntry.ToString());
            nrOfProdBatches = SystemDoubleConvertor.ObjectToDoubleOrNull(selectedProdOrderRow[0][PmxProductionOrderTable.Columns.NrOfProdBatches.NAME]);
            double? prodBatchSize = SystemDoubleConvertor.ObjectToDoubleOrNull(selectedProdOrderRow[0][PmxProductionOrderTable.Columns.ProdBatchSize.NAME]);

            /* BOYUM ROPI 002 START */
            DataSet dsCustMsg = null;

            query = @"SELECT
	                    CASE WHEN 
		                    SUM(""WOR1"".""PlannedQty"") 
		                    >
		                    SUM(CASE WHEN ""Free"".""AvailableQty"" > ""WOR1"".""PlannedQty"" THEN ""WOR1"".""PlannedQty"" ELSE ISNULL(""Free"".""AvailableQty"", 0) END) 
	                    THEN 'There is not enough quantity in the input location!'
	                    ELSE ''
	                    END as ""CheckQuantities""
                    FROM
	                    ""WOR1""
	                    INNER JOIN ""OWOR"" ON ""OWOR"".""DocEntry"" = ""WOR1"".""DocEntry""
	                    INNER JOIN ""PMX_OITM_MANAGED_BY_PMX"" ""po_oitm"" WITH (NOLOCK) ON	""po_oitm"".""ItemCode"" = ""OWOR"".""ItemCode""
	                    INNER JOIN ""PMX_OITM_MANAGED_BY_PMX"" WITH (NOLOCK) ON ""PMX_OITM_MANAGED_BY_PMX"".""ItemCode"" = ""WOR1"".""ItemCode""
	                    INNER JOIN ""PMX_OSPL"" ON ""OWOR"".U_PMX_PLCD = ""PMX_OSPL"".""Code""
	                    LEFT JOIN ""OUOM"" ON	""OUOM"".""UomEntry"" = ""PMX_OITM_MANAGED_BY_PMX"".""IUoMEntry"" AND ""OUOM"".""UomEntry"" <> -1
	                    LEFT JOIN ( ""PMX_INVT"" LEFT JOIN ""PMX_ITRI"" ON ""PMX_ITRI"".""InternalKey"" = ""PMX_INVT"".""ItemTransactionalInfoKey"" ) ON
		                    ""PMX_INVT"".""ItemCode"" = ""WOR1"".""ItemCode""
		                    AND ""PMX_INVT"".""StorLocCode"" = N'01PLBIN_CP1'
		                    AND ( ""PMX_INVT"".""LogUnitIdentKey"" IS NULL
			                    OR ""PMX_INVT"".""LogUnitIdentKey"" NOT IN (
				                    SELECT
					                    ""PMX_POPC"".""LogUnitIdentKey""
				                    FROM
					                    ""PMX_POPC"" 
				                    ) 
			                    )
			                    AND ( ""PMX_ITRI"".""BestBeforeDate"" IS NULL
			                    OR ( CASE
				                    WHEN ""WOR1"".""U_PMX_BBDO"" = N'P' THEN CURRENT_TIMESTAMP
				                    WHEN ""WOR1"".""U_PMX_BBDO"" = N'B' 
				                    THEN
					                    ISNULL( ""OWOR"".""U_PMX_BBDT"", ""OWOR"".""DueDate"" )
				                    ELSE 
					                    ""OWOR"".""DueDate""
				                    END < ""PMX_ITRI"".""BestBeforeDate"" ) 
		                    )
	                    LEFT JOIN (
		                    SELECT 
			                    ""ItemCode"", ""StorLocCode"", SUM(Quantity) as ""AvailableQty""
		                    FROM
			                    PMX_FREE_STOCK 
		                    GROUP BY ""ItemCode"", ""StorLocCode""
	                    ) as ""FREE""
	                    ON ""FREE"".""ItemCode"" = ""PMX_OITM_MANAGED_BY_PMX"".""ItemCode"" AND ""PMX_OSPL"".""InputLocationCode"" = ""FREE"".""StorLocCode""
                    WHERE ""WOR1"".""DocEntry"" = " + productionOrderDocEntry.ToString() + @" AND ""PMX_OITM_MANAGED_BY_PMX"".""InvntItem"" = 'Y' AND ISNULL(""PMX_OITM_MANAGED_BY_PMX"".""U_PMX_UFTR"", 'N' ) = 'N' AND ""WOR1"".""U_PMX_HBLU"" = 'N'";

            dsCustMsg = m_sboProviderService.RunView(false, null, null, query);

            if (dsCustMsg.Tables[0].Rows[0][0].ToString().Length > 0)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                this.DefaultCultureInfo, BuildParamCollection(
                    "MessageKey", "There is not enough quantity in the input location!",
                    "ShowButton", true
                ));
                msg = WaitForMessage();

            }
            /* BOYUM ROPI 002 STOP */



            goto Step_GetItemsForProductionOrder;


        #endregion

        Step_GetItemsForProductionOrder:

            #region GET ITEMS FOR PRODUCTION ORDER

            productionOrderLineNum = null;

            //query = Produmex.Sbo.Logex.Data.SboCommands.GetQueryGetProductionIngredients( productionOrderDocEntry.Value );
            //dsProductionOrder = m_sboProviderService.RunView( false, null, null, query );

            DataSet dsItems = new DataSet();
            DataSet dsPreparedItems = new DataSet();
            m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,

            delegate (PmxDbConnection dbConn, object[] parameters)
            {
                query = BuildQuery.GetProductionIngredientsStatus(productionOrderDocEntry.Value, weighingInputLocationCode, null, true, nrOfProdBatches.HasValue, false, lockPickedItemsForPickTo, dbConn.DbTool);
                using (DbDataReader rdr = dbConn.CreateCommand(query).ExecuteReader())
                {
                    DataTable tbl = dsItems.Tables.Add();
                    tbl.Load(rdr);
                }

                query = BuildQuery.GetPrepairedQuantities(productionOrderDocEntry.Value, weighingInputLocationCode, null, true, dbConn.DbTool);
                using (DbDataReader rdr = dbConn.CreateCommand(query).ExecuteReader())
                {
                    DataTable tbl = dsPreparedItems.Tables.Add();
                    tbl.Load(rdr);
                }
                return null;
            });



            dsItems.Tables[0].Columns.Add(SelectItemToPrepareDataSet.Columns.TOTAL_PREPARED_QTY.Name, typeof(double));
            dsItems.Tables[0].Columns.Add(SelectItemToPrepareDataSet.Columns.INCOMPLETE_WEIGHING.Name, typeof(string));

            //Store if everything has been weighed.
            //If so, extra processing will happen
            bool weighingComplete = true;

            foreach (DataRow itemRow in dsItems.Tables[0].Rows)
            {
                string itemCode = Convert.ToString(itemRow[SelectItemToPrepareDataSet.Columns.ITEM_CODE.Name]);
                int lineNum = Convert.ToInt32(itemRow[SelectItemToPrepareDataSet.Columns.LINE_NUM.Name]);
                double qty = 0.0;

                bool incompleteWeighing = false;

                foreach (DataRow preparedItemRow in dsPreparedItems.Tables[0].Rows)
                {
                    if (itemCode.CompareTo(preparedItemRow[SelectItemToPrepareDataSet.Columns.ITEM_CODE.Name]) == 0 &&
                        lineNum == Convert.ToInt32(preparedItemRow[SelectItemToPrepareDataSet.Columns.LINE_NUM.Name]))
                    {
                        qty += Convert.ToDouble(preparedItemRow[SelectItemToPrepareDataSet.Columns.QUANTITY.Name]);

                        double preparedQtyByBatch = Convert.ToDouble(preparedItemRow[SelectItemToPrepareDataSet.Columns.TOTAL_PREPARED_QTY_BY_BATCH.Name]);
                        double qtyPlannedBatch = Convert.ToDouble(preparedItemRow[SelectItemToPrepareDataSet.Columns.QTY_PLANNED_BY_BATCH.Name]);
                        double preparedQtyByBatchLower = qtyPlannedBatch * (100.0 - SystemDoubleConvertor.ObjectToDouble(itemRow[SelectItemToPrepareDataSet.Columns.TOLERANCE_PCT.Name], 0, 0)) / 100.0;

                        if (!incompleteWeighing &&
                            preparedQtyByBatch < preparedQtyByBatchLower)
                        {
                            incompleteWeighing = true;
                            weighingComplete = false;
                        }
                    }
                }
                itemRow[SelectItemToPrepareDataSet.Columns.TOTAL_PREPARED_QTY.Name] = qty;
                itemRow[SelectItemToPrepareDataSet.Columns.INCOMPLETE_WEIGHING.Name] = incompleteWeighing ? SboBooleanColumnValues.Y.VALUE : null;

            }

            if (!screenRefresh)
            {
                session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectItemToPrepareScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_ITEM_TO_WEIGH",
                    "ButtonKey", weighingComplete ? "MSG_BUTTON_RETURN_UNWEIGHED_ITEMS" : null,
                    "ItemsDS", dsItems),
                    WorkflowId,
                    nameof(ComponentWeighingProductionScript_Screens.SelectItemToPrepareScreen1));
            }
            else
            {
                session.UpdateScreen(BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "ButtonKey", weighingComplete ? "MSG_BUTTON_RETURN_UNWEIGHED_ITEMS" : null,
                    "ItemsDS", dsItems));
                screenRefresh = false;
            }
            msg = WaitForMessage();
            goBack = false;
            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;
                goto Step_SelectProductionOrder;
            }
            if (msg.Name.EndsWith(".RefreshRequested"))
            {
                screenRefresh = true;
                goto Step_GetItemsForProductionOrder;
            }

            if (msg.Name.EndsWith(".TimeRequested"))
            {
                returnAfterTimeRegistration = "Step_GetItemsForProductionOrder";
                goto Step_TimeRegistration;
            }

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                goto Step_MoveItemToRestStart;
            }

            if (!msg.Name.EndsWith(".ItemSelected"))
            {
                initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                goto Step_GetItemsForProductionOrder;
            }

            // User selected an item
            string selectedItemCode = ExtractParameter<string>(msg.Parameters, "itemCode");
            int? selectedItriKey = ExtractParameter<int?>(msg.Parameters, "itriKey");
            int? selectedLUID = ExtractParameter<int?>(msg.Parameters, "luid");
            productionOrderLineNum = ExtractParameter<int?>(msg.Parameters, "lineNum");

            goto Step_InteractiveWeighing;

        #endregion

        Step_MoveItemToRestStart:

            #region MOVE ITEMS TO REST LOCATION

            //there are ordered items, ask user if he wants to go back
            session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "MessageKey", "MSG_QUESTION_MOVE_UNWEIGHED_ITEMS",
                    "TitleKey", "MSG_TITLE_QUESTION_MOVE_UNWEIGHED_ITEMS",
                    "ButtonKeyNo", "MSG_QUESTION_MOVE_UNWEIGHED_ITEMS_BTN_NO",
                    "ButtonKeyYes", "MSG_QUESTION_MOVE_UNWEIGHED_ITEMS_BTN_YES"
                    ),
                WorkflowId,
                nameof(ComponentWeighingProductionScript_Screens.DecisionScreen1));
            msg = WaitForMessage();

            if (msg.Name.EndsWith(".No"))
            {
                //Go back
                goto Step_GetItemsForProductionOrder;
            }

            //We can move them
            Collection<string> listOfItemCodes = new Collection<string>();

            foreach (DataRow itemRow in dsItems.Tables[0].Rows)
            {
                string itemCode = Convert.ToString(itemRow[SelectItemToPrepareDataSet.Columns.ITEM_CODE.Name]);
                if (!listOfItemCodes.Contains(itemCode))
                {
                    listOfItemCodes.Add(itemCode);
                }
            }

        Step_MoveItemToRest:



            try
            {
                m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                     delegate (PmxDbConnection dbConn, object[] parameters)
                     {

                         PmxOseProductionLineProvider prov = new PmxOseProductionLineProvider(dbConn);
                         prov.MoveStockToRestLocationFromPickToLocation(productionLineCode, productionOrderDocEntry, listOfItemCodes);

                         return null;
                     });
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
                {
                    // Inform the user with the error
                    session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true),
                        WorkflowId,
                        nameof(ComponentWeighingProductionScript_Screens.ShowMessageScreen1));
                    WaitForMessage();

                }
                else
                {
                    // Inform the user with the error
                    session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true),
                        WorkflowId,
                        nameof(ComponentWeighingProductionScript_Screens.ShowMessageScreen2));
                    WaitForMessage();
                }

                //retry?
                //ask user if he wants to retry
                session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_RETRY_UPDATE",
                        "MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"),
                    WorkflowId,
                    nameof(ComponentWeighingProductionScript_Screens.DecisionScreen2));
                msg = WaitForMessage();

                if (msg.Name.ToLower().EndsWith(".yes"))
                {
                    goto Step_MoveItemToRest;
                }


                goto Step_GetItemsForProductionOrder;
            }



            goto Step_SelectProductionOrder;

        #endregion

        Step_InteractiveWeighing:

            #region Step_InteractiveWeighing

            DataRow[] selectedItemRows = dsItems.Tables[0].Select(String.Format("{0} = {1} AND {2} {3} {4}  AND {5} {6} {7} AND {8} = {9}",
                     DataSetCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.ITEM_CODE.Name),
                     DataSetCommandHelper.FormatAndEscapeColumnValue(selectedItemCode, this.DefaultCultureInfo),
                     DataSetCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.ITRI_KEY.Name),
                     selectedItriKey.HasValue ? "=" : "",
                     selectedItriKey.HasValue ? DataSetCommandHelper.FormatAndEscapeColumnValue(selectedItriKey.Value, this.DefaultCultureInfo) : "IS NULL",
                     DataSetCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.LUID.Name),
                     selectedLUID.HasValue ? "=" : "",
                     selectedLUID.HasValue ? DataSetCommandHelper.FormatAndEscapeColumnValue(selectedLUID.Value, this.DefaultCultureInfo) : "IS NULL",
                     DataSetCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.LINE_NUM.Name),
                     productionOrderLineNum ?? -1
                    ));

            if (selectedItemRows.Length < 1)
            {
                initialErrorKey = "MSG_ERROR";
                goto Step_GetItemsForProductionOrder;
            }

            int uomDecimals = SystemInt32Convertor.ObjectToInt(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.UOM_DECIMALS.Name], 0, 0);
            string uom = Convert.ToString(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.UOM.Name]);

            DataTable tblNextProdBatchInfo = new DataTable();
            int? prodBatchInternalKey = null;
            int? prodBatchSeqNr = null;
            double qtyPlanned = 0;
            double qtyPrepared = 0;
            double qtyIssued = 0;
            if (nrOfProdBatches.HasValue)
            {
                m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                delegate (PmxDbConnection dbConn, object[] parameters)
                {
                    query = BuildQuery.GetNextProductionBatchInfo(productionOrderDocEntry.Value, selectedItemCode, dbConn.DbTool);
                    using (DbDataReader rdr = dbConn.CreateCommand(query).ExecuteReader())
                    {
                        tblNextProdBatchInfo.Load(rdr);
                    }

                    qtyPlanned = Convert.ToDouble(tblNextProdBatchInfo.Rows[0][PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME]);
                    qtyPrepared = SystemDoubleConvertor.ObjectToDouble(tblNextProdBatchInfo.Rows[0][PmxInventoryTotalTable.NAME + "_" + PmxInventoryTotalTable.Columns.Quantity.NAME], 0, 0);
                    qtyIssued = 0;

                    string weighingStatus = Convert.ToString(tblNextProdBatchInfo.Rows[0][PmxProductionOrderBatchTable.Columns.WeighingStatus.NAME]);
                    if (String.IsNullOrEmpty(weighingStatus))
                    {
                        prodBatchSeqNr = 1;
                        qtyPrepared = 0;
                    }
                    else if (weighingStatus == PmxProductionStatusColumnValues.Planned.VALUE
                         || (weighingStatus == PmxProductionStatusColumnValues.Started.VALUE && qtyPrepared < Convert.ToDouble(tblNextProdBatchInfo.Rows[0]["lowerTolQty"])))
                    {
                        prodBatchInternalKey = Convert.ToInt32(tblNextProdBatchInfo.Rows[0][PmxProductionOrderBatchTable.Columns.InternalKey.NAME]);
                        prodBatchSeqNr = Convert.ToInt32(tblNextProdBatchInfo.Rows[0][PmxProductionOrderBatchTable.Columns.SeqNr.NAME]);
                    }
                    else
                    {
                        prodBatchSeqNr = Convert.ToInt32(tblNextProdBatchInfo.Rows[0][PmxProductionOrderBatchTable.Columns.SeqNr.NAME]) + 1;
                        qtyPrepared = 0;
                    }

                    if (!prodBatchInternalKey.HasValue)
                    {
                        PmxProductionOrderBatchProvider pbProv = new PmxProductionOrderBatchProvider(dbConn);
                        using (PmxProductionOrderBatch pb = pbProv.GetNewBO())
                        {
                            pb.DocEntry = productionOrderDocEntry.Value;
                            pb.SeqNr = prodBatchSeqNr.Value;
                            pb.PlannedQty = Convert.ToDouble(tblNextProdBatchInfo.Rows[0][PmxProductionOrderTable.Columns.ProdBatchSize.NAME]);
                            pb.WeighingStatus = PmxProductionStatusCodes.Started;

                            pbProv.AddBO(pb);
                        }
                    }
                    else if (weighingStatus != PmxProductionStatusColumnValues.Started.VALUE)
                    {
                        PmxProductionOrderBatchProvider pbProv = new PmxProductionOrderBatchProvider(dbConn);
                        using (PmxProductionOrderBatch pb = pbProv.GetBO(prodBatchInternalKey.Value))
                        {
                            pb.WeighingStatus = PmxProductionStatusCodes.Started;

                            pbProv.UpdateBO(pb, false, false);
                        }
                    }

                    return null;
                });
            }
            else
            {
                prodBatchInternalKey = null;
                prodBatchSeqNr = null;
                qtyPlanned = Convert.ToDouble(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.QTY_PLANNED.Name]);
                qtyPrepared = 0;
                qtyIssued = SystemDoubleConvertor.ObjectToDouble(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.QTY_ISSUED.Name], 0, 0);
            }

            DataSet dsTargetLuid = new DataSet();
            DataTable tblTargetLuid = dsTargetLuid.Tables.Add();
            tblTargetLuid.Columns.Add("InternalKey", typeof(int));
            tblTargetLuid.Columns.Add("SSCC", typeof(string));
            tblTargetLuid.Columns.Add("Quantity", typeof(double));
            tblTargetLuid.Columns.Add("UomDecimals", typeof(int));
            tblTargetLuid.Columns.Add("Uom", typeof(string));
            tblTargetLuid.Rows.Add(null, "<new>");

            PmxDictionary<int, DataRow> targetLuids = new PmxDictionary<int, DataRow>();
            foreach (DataRow row in dsPreparedItems.Tables[0].Select(String.Format("{0} = {1} AND {2} = {3} AND {4} = {5}",
                    DataSetCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.ITEM_CODE.Name),
                    DataSetCommandHelper.FormatAndEscapeColumnValue(selectedItemCode, this.DefaultCultureInfo),
                    DataSetCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.STORAGE_LOCATION.Name),
                    DataSetCommandHelper.FormatAndEscapeColumnValue(weighingOutputLocationCode, this.DefaultCultureInfo),
                    DataSetCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.LINE_NUM.Name),
                    DataSetCommandHelper.FormatAndEscapeColumnValue(productionOrderLineNum ?? -1, this.DefaultCultureInfo))))
            {
                double qty = Convert.ToDouble(row[SelectItemToPrepareDataSet.Columns.QUANTITY.Name]);
                if (!Tester.IsNullOrEmpty(row[SelectItemToPrepareDataSet.Columns.LUID.Name])
                    && (!nrOfProdBatches.HasValue || prodBatchSeqNr.Value.Equals(row[SelectItemToPrepareDataSet.Columns.PROD_BATCH_SEQ_NR.Name])))
                {
                    int luid = Convert.ToInt32(row[SelectItemToPrepareDataSet.Columns.LUID.Name]);
                    DataRow targetLuidRow;
                    if (!targetLuids.TryGetValue(luid, out targetLuidRow))
                    {
                        targetLuids.Add(luid, tblTargetLuid.Rows.Add(
                            luid,
                            row[SelectItemToPrepareDataSet.Columns.SSCC.Name],
                            row[SelectItemToPrepareDataSet.Columns.QUANTITY.Name],
                            uomDecimals,
                            uom));
                    }
                    else
                    {
                        targetLuidRow["Quantity"] = Convert.ToDouble(targetLuidRow["Quantity"]) + qty;
                    }
                }
                if (!nrOfProdBatches.HasValue)
                {
                    qtyPrepared += qty;
                }
            }

            double tolPct = SystemDoubleConvertor.ObjectToDouble(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.TOLERANCE_PCT.Name], 0, 0);
            double[] weighingTarget = new double[] {
                Math.Max( 0.0, qtyPlanned - qtyPrepared - qtyIssued ),
                qtyPlanned * tolPct / 100.0 };

            // Find scale to use
            string scaleProvType = null;
            string scaleSettings = null;
            int scaleDecimals = 6;
            double scaleMaxWeight = 0.0;
            m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
            delegate (PmxDbConnection dbConn, object[] parameters)
            {
                DbCommand dbCmd = dbConn.CreateCommand(BuildQuery.GetThinClientScales(session.DeviceInstanceId, dbConn.DbCultureInfo));
                using (DbDataReader rdr = dbCmd.ExecuteReader())
                {
                    while (rdr.Read())
                    {
                        double maxWeight = SystemDoubleConvertor.ObjectToDouble(rdr[PmxScaleDefinitionTable.Columns.MaxWeight.NAME], 1e9, 1e9);
                        if (maxWeight < qtyPlanned && !String.IsNullOrEmpty(scaleSettings))
                        {
                            break;
                        }
                        scaleProvType = Convert.ToString(rdr[PmxScaleDefinitionTable.Columns.ProviderType.NAME]);
                        scaleSettings = Convert.ToString(rdr[PmxScaleDefinitionTable.Columns.Settings.NAME]);
                        scaleDecimals = SystemInt32Convertor.ObjectToInt(rdr[PmxScaleDefinitionTable.Columns.NrOfDecimals.NAME], 0, 0);
                        scaleMaxWeight = maxWeight;
                    }
                }

                return null;
            });

            string baseDocInfo = this.ResourceStore.GetPropertyValue<string>("SlimScreen.InteractiveScaleScreen.LblBaseDocInfo", "FormatForBatch{0}of{1}");

            session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.IInteractiveScaleScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_WEIGH_ITEM_ROPI",
                "BaseDocInfo", nrOfProdBatches.HasValue ? String.Format(baseDocInfo, prodBatchSeqNr.Value, nrOfProdBatches.Value) : "",
                "ItemCode", selectedItemCode,
                "ItemDescr", Convert.ToString(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.ITEM_DESCR.Name]),
                "BatchNum", Convert.ToString(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.BATCH_NUMBER.Name]),
                "BestBeforeDate", String.Format(this.DefaultCultureInfo.Get(), "{0:d}", selectedItemRows[0][SelectItemToPrepareDataSet.Columns.BEST_BEFORE_DATE.Name]),
                "Uom", uom,
                "UomDecimals", uomDecimals,
                "QtyPlanned", qtyPlanned,
                "WeighingTarget", weighingTarget,
                "QtyAvailable", SystemDoubleConvertor.ObjectToDouble(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.QUANTITY.Name], 0, 0),
                "TargetLuidDS", dsTargetLuid,
                "ScaleProviderType", scaleProvType,
                "ScaleDecimals", scaleDecimals,
                "ScaleMaxWeight", scaleMaxWeight,
                "ScaleSettings", scaleSettings,
                /*                "ButtonPartialKey", buttonLocalizationForPartialWeighingButton, */
                "ButtonCompleteKey", "MSG_BUTTON_COMPLETE_WEIGHING"),
                WorkflowId,
                nameof(ComponentWeighingProductionScript_Screens.InteractiveScaleScreen1));
            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_GetItemsForProductionOrder;
            }
            if (msg.Name.EndsWith(".WeighingDone"))
            {
                double weight = ExtractParameter<double>(msg.Parameters, "weight");
                bool partial = ExtractParameter<bool>(msg.Parameters, "partial");
                bool theoQtyCorrection = ExtractParameter<bool>(msg.Parameters, "theoQtyCorrection");
                int? targetLuid = ExtractParameter<int?>(msg.Parameters, "targetLuid");

                // ROPIDEV
                if (weight < 10)
                {
                    Message msgVali = null;
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                           this.DefaultCultureInfo, BuildParamCollection(
                                "MessageKey", "Quantity is less than the necessary quantity: " + weight.ToString() + "",
                                "ShowButton", true
                                ));
                    msgVali = WaitForMessage();

                    goto Step_InteractiveWeighing;
                }



                int dstLuid = 0;
                try
                {
                    m_sboProviderService.InvokeMethodWithDbConnection<object>(theoQtyCorrection, false, null, null,  // Use SBO when theoQtyCorrection is needed
                    delegate (PmxDbConnection dbConn, object[] parameters)
                    {
                        // If theoretical qty correction is allowed, check how much we really do have to correct
                        double currentQtyAvailable = 0.0;
                        double theoQtyDifference = 0.0;
                        if (theoQtyCorrection)
                        {
                            DataTable tbl = new DataTable();

                            // Refetch available qty
                            BuildQuery.IngredientFilter filter = new BuildQuery.IngredientFilter();
                            filter.ItemCode = selectedItemCode;
                            filter.ItriKey = selectedItriKey;
                            filter.Luid = SystemInt32Convertor.ObjectToIntOrNull(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.LUID.Name]);
                            filter.QualityStatus = Convert.ToString(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.QUALITY_STATUS.Name]);


                            query = BuildQuery.GetProductionIngredientsStatus(productionOrderDocEntry.Value, weighingInputLocationCode, filter, true, nrOfProdBatches.HasValue, false, lockPickedItemsForPickTo, dbConn.DbTool);
                            using (DbDataReader rdr = dbConn.CreateCommand(query).ExecuteReader())
                            {
                                tbl.Load(rdr);
                            }
                            foreach (DataRow row in tbl.Rows)
                            {
                                currentQtyAvailable += Convert.ToDouble(row[SelectItemToPrepareDataSet.Columns.QUANTITY.Name]);
                            }
                            theoQtyDifference = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(weight - currentQtyAvailable, uomDecimals);
                            if (theoQtyDifference >= SboQuantity.Epsilon)
                            {
                                // Only move available qty
                                weight = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(currentQtyAvailable, uomDecimals);
                            }
                            else
                            {
                                // Not needed anymore
                                theoQtyCorrection = false;
                            }
                        }

                        // Determine dstLuid
                        if (targetLuid.HasValue)
                        {
                            dstLuid = targetLuid.Value;
                        }
                        else
                        {
                            PmxLogisticUnitIDProvider luidProv = new PmxLogisticUnitIDProvider(dbConn);
                            dstLuid = luidProv.GenerateNewLogisticUnit(null);
                        }

                        // Execute move of available qty
                        PmxMoveProvider moveProv = new PmxMoveProvider(dbConn);

                        List<MoveItemInfo> infos = new List<MoveItemInfo>();

                        MoveItemInfo info = new MoveItemInfo();
                        info.BaseType = SboObjectTypes.ProductionOrder;
                        info.BaseEntry = productionOrderDocEntry.Value;
                        info.BaseLine = Convert.ToInt32(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.LINE_NUM.Name]);
                        info.ItemCode = selectedItemCode;
                        info.ItemDescription = Convert.ToString(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.ITEM_DESCR.Name]);
                        info.ItemTransactionalInfoKey = selectedItriKey;
                        info.Quantity = weight;
                        info.QuantityPerUom = 1;
                        info.Uom = Convert.ToString(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.UOM.Name]);
                        info.SourceLogisticUnitIdentKey = SystemInt32Convertor.ObjectToIntOrNull(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.LUID.Name]);
                        info.SourceLocationCode = weighingInputLocationCode;
                        info.SourcePmxWhsCode = whsInfo.PmxWarehouseCode;
                        info.SourceQualityStatusCode = Convert.ToString(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.QUALITY_STATUS.Name]);
                        info.DestinationLogisticUnitIdentKey = dstLuid;
                        info.DestinationLocationCode = weighingOutputLocationCode;
                        info.DestinationPmxWhsCode = info.SourcePmxWhsCode;
                        info.DestinationQualityStatusCode = info.SourceQualityStatusCode;

                        //Catch weight
                        info.Uom2 = null;
                        info.QuantityUom2 = null;


                        infos.Add(info);

                        if (lockPickedItemsForPickTo)
                        {
                            //Remove locking
                            PmxInventoryLockingDetailProvider lockProv = new PmxInventoryLockingDetailProvider(dbConn);

                            using (CollectionWithDispose<PmxInventoryLockingDetail> lockingList = lockProv.GetInventoryLockingDetailsByProductAndBase(info.ItemCode, info.ItemTransactionalInfoKey, info.SourceLogisticUnitIdentKey, info.SourceQualityStatusCode, info.SourceLocationCode, PmxInventoryLockingLevel.ItemDetail,
                                info.BaseType, info.BaseEntry, info.BaseLine, info.SourcePmxWhsCode, true))
                            {
                                double qty = info.Quantity;
                                if (lockingList != null)
                                {
                                    foreach (PmxInventoryLockingDetail locking in lockingList)
                                    {
                                        if (locking.Quantity <= qty)
                                        {
                                            qty -= locking.Quantity;
                                            lockProv.DeleteBO(locking);
                                        }
                                        else
                                        {
                                            locking.Quantity -= qty;
                                            lockProv.UpdateBO(locking, false, false);
                                            break;
                                        }

                                        if (qty <= 0)
                                        {
                                            break;
                                        }
                                    }
                                }
                            }
                        }

                        moveProv.MakeMoveFromMoveItemInfoList(infos);

                        if (theoQtyCorrection)
                        {
                            PmxProductionReceiptProvider receiptProv = new PmxProductionReceiptProvider((SboConnection)dbConn);
                            using (PmxProductionReceipt receipt = receiptProv.GetNewBO())
                            {
                                //receipt.DocDate = DateTime.Now;
                                //receipt.DocDueDate = DateTime.Now;

                                DocumentLineInfo receiptLine = new DocumentLineInfo();
                                receiptLine.ItemCode = info.ItemCode;
                                receiptLine.ItemTransactionalInfoKey = info.ItemTransactionalInfoKey;

                                PmxItemTransactionalInfoProvider itriProv = new PmxItemTransactionalInfoProvider(dbConn);

                                if (info.ItemTransactionalInfoKey.HasValue)
                                {
                                    PmxItemTransactionalInfo itriInfo = itriProv.GetBO(info.ItemTransactionalInfoKey.Value);
                                    receiptLine.BatchNumber1 = itriInfo.BatchNumber1;
                                    receiptLine.BatchNumber2 = itriInfo.BatchNumber2;
                                    receiptLine.BestBeforeDate = itriInfo.BestBeforeDate;
                                }
                                receiptLine.QuantityPerUom = 1;
                                receiptLine.Quantity = theoQtyDifference;
                                receiptLine.LogisticUnitIdentKey = info.DestinationLogisticUnitIdentKey;
                                //receiptLine.SSCC = info.SSCC;
                                receiptLine.QualityStatusCode = info.DestinationQualityStatusCode;
                                receiptLine.StorageLocationCode = info.DestinationLocationCode;
                                receiptLine.WarehouseCode = whsInfo.SboWarehouseCode;

                                PmxProductionReceiptLine sboReceiptLine = DocumentInfoHelper.AddDocumentLineInfoToBO<PmxProductionReceipt, PmxProductionReceiptLine>(receiptLine, receipt, receiptProv);
                                sboReceiptLine.ProductionOrderDocEntry = productionOrderDocEntry;
                                sboReceiptLine.ProductionOrderLineNum = Convert.ToInt32(selectedItemRows[0][SelectItemToPrepareDataSet.Columns.LINE_NUM.Name]);

                                receiptProv.AddBO(receipt);
                            }
                        }

                        if (!targetLuid.HasValue)
                        {
                            PmxProductionOrderPreparedComponentProvider pcProv = new PmxProductionOrderPreparedComponentProvider(dbConn);
                            using (PmxProductionOrderPreparedComponent newPc = pcProv.GetNewBO())
                            {
                                newPc.ProdOrderDocEntry = productionOrderDocEntry.Value;
                                newPc.ProdBatchSeqNr = prodBatchSeqNr;
                                newPc.LogUnitIdentKey = dstLuid;
                                newPc.ItemCode = selectedItemCode;
                                newPc.Quantity = weight;
                                newPc.ProdOrderLineNum = productionOrderLineNum;

                                pcProv.AddBO(newPc);
                            }
                        }
                        else
                        {
                            CollectionWithDispose<PmxProductionOrderPreparedComponent> comps = new CollectionWithDispose<PmxProductionOrderPreparedComponent>();
                            string whereClause = BuildQuery.GetProductionOrderPreparedComponentWhereClause(productionOrderDocEntry.Value, prodBatchSeqNr, dstLuid, selectedItemCode, productionOrderLineNum, dbConn);
                            PmxInternalBusinessObjectHelper.GetBOsByWhereClause(s_log, dbConn, typeof(PmxProductionOrderPreparedComponent), whereClause, null, true, comps);

                            PmxProductionOrderPreparedComponentProvider pcProv = new PmxProductionOrderPreparedComponentProvider(dbConn);

                            if (comps.Count > 0)
                            {
                                comps[0].Quantity += weight;
                                pcProv.UpdateBO(comps[0], false, false);
                            }
                            else
                            {
                                using (PmxProductionOrderPreparedComponent newPc = pcProv.GetNewBO())
                                {
                                    newPc.ProdOrderDocEntry = productionOrderDocEntry.Value;
                                    newPc.ProdBatchSeqNr = prodBatchSeqNr;
                                    newPc.LogUnitIdentKey = dstLuid;
                                    newPc.ItemCode = selectedItemCode;
                                    newPc.Quantity = weight;
                                    newPc.ProdOrderLineNum = productionOrderLineNum;

                                    pcProv.AddBO(newPc);
                                }
                            }
                        }

                        //PmxWeightRegistrationProvider wrProv = new PmxWeightRegistrationProvider( dbConn );
                        //using ( PmxWeightRegistration wr = wrProv.GetNewBO() )
                        //{
                        //    wr.ObjType = SboObjectTypes.ProductionOrder;
                        //    wr.DocEntry = productionOrderDocEntry.Value;
                        //    wr.LineNum = Convert.ToInt32( selectedItemRows[ 0 ][ SelectItemToPrepareDataSet.Columns.LINE_NUM.Name ] );
                        //    wr.ItemCode = selectedItemCode;
                        //    wr.NetQuantity = weight;

                        //    wrProv.AddBO( wr );
                        //}

                        return null;
                    });
                }
                catch (ProdumexException ex)
                {
                    s_log.Error(ex.ToString());
                    session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true),
                        WorkflowId,
                        nameof(ComponentWeighingProductionScript_Screens.ShowMessageScreen3));
                    msg = WaitForMessage();
                    initialErrorKey = null;
                    goto Step_InteractiveWeighing;
                }
                catch (Exception ex)
                {
                    s_log.Error(ex.ToString());
                    session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true),
                        WorkflowId,
                        nameof(ComponentWeighingProductionScript_Screens.ShowMessageScreen4));
                    msg = WaitForMessage();
                    initialErrorKey = null;
                    goto Step_InteractiveWeighing;
                }

                if (!partial || (partial && !printLabelsOnPartialWeighingOption.Equals(PmxYesNoAskOptionCodes.NO)))
                {
                    bool print = true;
                    if (partial && printLabelsOnPartialWeighingOption.Equals(PmxYesNoAskOptionCodes.ASK))
                    {
                        session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "MSG_QUESTION_PRINT_LABEL_FOR_PARTIAL_WEIGHING",
                            "TitleKey", "MSG_TITLE_PRINT_LABEL_FOR_PARTIAL_WEIGHING"
                            ),
                        WorkflowId,
                        nameof(ComponentWeighingProductionScript_Screens.DecisionScreen3));
                        msg = WaitForMessage();
                        initialErrorKey = null;

                        print = msg.Name.EndsWith(".Yes");

                    }

                    while (print)
                    {
                        int numberOfReports = 0;
                        bool askReprint = true;

                        try
                        {
                            //trigger event
                            numberOfReports += m_sboProviderService.InvokeMethodWithDbConnectionOnThread<int>(false, true, null, null, delegate (PmxDbConnection conn, object[] parameters)
                            {
                                PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                                return printProv.PrintReportsForEvent(PmxPrintReportEventType.WarehouseCreatedLogisticUnit, weighingOutputLocationCode, session.DeviceInstanceId, dstLuid, out askReprint, stepHelper.GetPrinterFromSelectPrinterScreen);
                            });
                        }
                        catch (Produmex.Foundation.ProdumexException ex)
                        {
                            s_log.Error(ex.ToString());
                            session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection(
                                    "InitialErrorKey", initialErrorKey,
                                    "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                    "MessageAddendum", ex.Message,
                                    "PmxError", ex.Error,
                                    "ShowButton", true),
                                WorkflowId,
                                nameof(ComponentWeighingProductionScript_Screens.ShowMessageScreen5));
                            msg = WaitForMessage();
                            initialErrorKey = null;
                        }
                        catch (Exception ex)
                        {
                            s_log.Error(ex.ToString());
                            session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection(
                                    "InitialErrorKey", initialErrorKey,
                                    "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                    "MessageAddendum", ex.Message,
                                    "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                                    "ShowButton", true),
                                WorkflowId,
                                nameof(ComponentWeighingProductionScript_Screens.ShowMessageScreen6));
                            msg = WaitForMessage();
                            initialErrorKey = null;
                        }
                        if (numberOfReports < 1 || !askReprint)
                        {
                            break;
                        }

                        session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
                                "TitleKey", "MSG_TITLE_REPRINT_LABEL"
                                ),
                            WorkflowId,
                            nameof(ComponentWeighingProductionScript_Screens.DecisionScreen4));
                        msg = WaitForMessage();

                        if (msg.Name.EndsWith(".No"))
                        {
                            break;
                        }

                    }
                }

                goto Step_GetItemsForProductionOrder;
            }


        #endregion

        Step_TimeRegistration:

            #region TimeRegistration


            m_timeRegistrationSubFlow.BindParameters(
                this.DefaultCultureInfo,
                this.PmxOseCompany,
                new ReadOnlyBinder<string>("ObjectType", SboObjectTypes.ProductionOrder),
                new ReadOnlyBinder<int>("DocEntry", productionOrderDocEntry.Value),
                new ReadOnlyBinder<int?>("LineNum", null),
                new ReadOnlyBinder<string>("ItemCode", null),
                new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested")

                );

            this.StartSubFlowAndWait(m_timeRegistrationSubFlow, true);


            //go back to flow
            switch (returnAfterTimeRegistration)
            {
                case "Step_GetItemsForProductionOrder":
                    returnAfterTimeRegistration = null;
                    goto Step_GetItemsForProductionOrder;
                default:
                    returnAfterTimeRegistration = null;
                    goto Step_SelectProductionOrder;
            }

            #endregion

        }

        #endregion

        #region HELPERS

        /// <summary>
        /// Executes the get item info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetCachedItemInfo((string)parameters[0]);
        }

        /// <summary>
        /// Executes the get PMX warehouse code.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
        {
            PmxOseProvider oseProv = new PmxOseProvider(conn);
            return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
        }

        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            return (ds != null && ds.Tables.Count > 0) ? ds.Tables[0].Rows.Count : 0;
        }

        #endregion

        #region QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        private class BuildQuery
        {



            /// <summary>
            /// Gets all the production lines.
            /// </summary>
            /// <param name="warehouseCode">The warehouse code.</param>
            /// <returns>The sql statement to get all the production lines</returns>
            public static string GetProductionLines(string warehouseCode, string deviceID, string Zone)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.InputLocationCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.PickToLocationCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.LockPickedItemsForPickTo.NAME));



                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.OutputLocationCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.RestLocationCode.NAME));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));



                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.Code.NAME);

                // strBuilder.Append(" WHERE  ");
                /* BOYUM ROPI 001 START */
                strBuilder.Append(" WHERE \"PMX_OSEL\".\"ParentCode\" = '" + Zone + "' AND  ");
                /* BOYUM ROPI 001 START */

                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                if (!String.IsNullOrEmpty(warehouseCode) || !String.IsNullOrEmpty(deviceID))
                {
                    strBuilder.Append(" AND ( ");
                    if (!String.IsNullOrEmpty(warehouseCode))
                    {
                        SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));
                    }
                    if (!String.IsNullOrEmpty(deviceID))
                    {
                        if (!String.IsNullOrEmpty(warehouseCode)) strBuilder.Append(" OR ");

                        strBuilder.AppendFormat("{0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} )",
                            SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.PmxWarehouseCode.NAME),
                            SqlCommandHelper.EscapeTableName(PmxThinClientWarehouseTable.NAME),
                            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.ParentCode.NAME),
                            SqlCommandHelper.FormatAndEscapeColumnValue(deviceID, CultureInfo.InvariantCulture));
                    }
                    strBuilder.Append(" )");
                }


                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));


                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the production orders.
            /// </summary>
            /// <param name="productionLineCode">The production line code.</param>
            /// <param name="productionOrderDocEntry">The production order doc entry.</param>
            /// <returns></returns>
            public static string GetProductionOrders(string productionLineCode, int? productionOrderDocEntry, DbTool dbTool)
            {
                DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
                qryBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductionOrderDataSet.Columns.ITEMDESCRIPTION.Name);

                qryBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME, ProductionOrderDataSet.Columns.ITEMCODE.Name);

                qryBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name);

                qryBuilder.Append(", CAST(");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                qryBuilder.Append(" AS NVARCHAR) AS ");
                qryBuilder.Append(qryBuilder.AsIdentifier(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY_AS_STRING.Name));

                qryBuilder.Append(", CAST(");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME);
                qryBuilder.Append(" AS NVARCHAR) AS ");
                qryBuilder.Append(qryBuilder.AsIdentifier(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_NUM_AS_STRING.Name));

                qryBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Comments.NAME, ProductionOrderDataSet.Columns.COMMENTS.Name);

                qryBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME, ProductionOrderDataSet.Columns.DOC_NUM.Name);

                qryBuilder.Append(", ");
                qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME, ProductionOrderDataSet.Columns.PRODUCTION_LINE_CODE.Name));

                qryBuilder.Append(", ");
                qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionOrderStepListKey.NAME, ProductionOrderDataSet.Columns.STEP_LIST_KEY.Name));

                qryBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME, ProductionOrderDataSet.Columns.DUE_DATE.Name);

                qryBuilder.Append(", ");
                qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME, ProductionOrderDataSet.Columns.CUSTOMER_CODE.Name));

                qryBuilder.Append(", ");
                qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardName.NAME, ProductionOrderDataSet.Columns.CUSTOMER_DESCRIPTION.Name));

                qryBuilder.Append(", (");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
                qryBuilder.Append(" - ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);
                qryBuilder.Append(" - ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.RejectedQuantity.NAME);

                qryBuilder.Append(" ) AS ");
                qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name));

                qryBuilder.Append(", ");
                qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.InputLocationCode.NAME, ProductionOrderDataSet.Columns.INPUT_LOCATION_CODE.Name));

                qryBuilder.Append(", ");
                qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME, ProductionOrderDataSet.Columns.UOM.Name));

                qryBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Type.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name);
                qryBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionStatus.NAME, ProductionOrderDataSet.Columns.PMX_PRODUCTION_STATUS.Name);
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProdBatchSize.NAME, PmxProductionOrderTable.Columns.ProdBatchSize.NAME));
                qryBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.BatchNumber.NAME, ProductionOrderDataSet.Columns.PMX_BATCH_NUM.Name);

                qryBuilder.Append(" FROM ");
                qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));


                qryBuilder.Append("\r\n INNER JOIN ");
                qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
                qryBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
                qryBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Warehouse.NAME);

                qryBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
                qryBuilder.Append(" = ");
                qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));



                qryBuilder.Append(" INNER JOIN ");
                qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                qryBuilder.Append(qryBuilder.WITH_NOLOCK);
                qryBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);
                qryBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);

                qryBuilder.Append(" LEFT JOIN ");
                qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));
                qryBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardCode.NAME);
                qryBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME);

                qryBuilder.Append(" LEFT JOIN ");
                qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
                qryBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
                qryBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.Code.NAME);



                qryBuilder.Append(" WHERE ");
                qryBuilder.Append(" ( ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
                qryBuilder.Append(" IS NULL ");

                if (!string.IsNullOrEmpty(productionLineCode))
                {
                    qryBuilder.Append(" OR ");
                    SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
                    qryBuilder.Append(" = ");
                    qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionLineCode, CultureInfo.InvariantCulture));
                }
                qryBuilder.Append(" ) ");

                qryBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Status.NAME);
                qryBuilder.Append(" = ");
                qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboProductionOrderStatusColumnValues.Released.VALUE, CultureInfo.InvariantCulture));

                if (productionOrderDocEntry.HasValue)
                {
                    qryBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                    qryBuilder.Append(" = ");
                    qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry.Value, CultureInfo.InvariantCulture));

                }


                qryBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME);


                return qryBuilder.ToString();
            }

            internal class IngredientFilter
            {
                public string ItemCode;
                public int? ItriKey;
                public int? Luid;
                public string QualityStatus;
            }

            /// <summary>
            /// Gets the production ingredients status.
            /// </summary>
            /// <param name="prodOrderDocEntry">The prod order doc entry.</param>
            /// <param name="inputLocationCode">The input location code.</param>
            /// <param name="filter">The filter.</param>
            /// <param name="filterExpiredProducts">if set to <c>true</c> [filter expired products].</param>
            /// <param name="useProdBatches">if set to <c>true</c> [use prod batches].</param>
            /// <param name="hasToBeLinedUpItems">The has to be lined up items.</param>
            /// <param name="itemsAreLocked">if set to <c>true</c> [items are locked].</param>
            /// <param name="dbTool">The db tool.</param>
            /// <returns></returns>
            internal static string GetProductionIngredientsStatus(int prodOrderDocEntry, string inputLocationCode, IngredientFilter filter, bool filterExpiredProducts, bool useProdBatches, bool? hasToBeLinedUpItems, bool itemsAreLocked, DbTool dbTool)
            {
                DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

                string po_oitm = "po_oitm";

                qryBuilder.AppendFormat("SELECT {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, SelectItemToPrepareDataSet.Columns.LINE_NUM.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME, SelectItemToPrepareDataSet.Columns.ITEM_CODE.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemName.NAME, SelectItemToPrepareDataSet.Columns.ITEM_DESCR.Name));


                qryBuilder.AppendFormat(", " + qryBuilder.IFNULL + "({0}, {1}) AS {2} ",
                  SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME),
                  SqlCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.UOM.Name));

                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UomDecimals.NAME, SelectItemToPrepareDataSet.Columns.UOM_DECIMALS.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME, SelectItemToPrepareDataSet.Columns.QTY_PLANNED.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME, SelectItemToPrepareDataSet.Columns.TOLERANCE_PCT.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.IssuedQuantity.NAME, SelectItemToPrepareDataSet.Columns.QTY_ISSUED.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, SelectItemToPrepareDataSet.Columns.ITRI_KEY.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, SelectItemToPrepareDataSet.Columns.BATCH_NUMBER.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, SelectItemToPrepareDataSet.Columns.BEST_BEFORE_DATE.Name));

                if (!itemsAreLocked)
                {
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME, SelectItemToPrepareDataSet.Columns.QUANTITY.Name));
                }
                else
                {
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.Quantity.NAME, SelectItemToPrepareDataSet.Columns.QUANTITY.Name));
                }
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME, SelectItemToPrepareDataSet.Columns.QUALITY_STATUS.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, SelectItemToPrepareDataSet.Columns.LUID.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SelectItemToPrepareDataSet.Columns.STORAGE_LOCATION.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, SelectItemToPrepareDataSet.Columns.SSCC.Name));
                //strBuilder.AppendFormat( ", {0}",
                //    SqlCommandHelper.EscapeTableColumnNameWithAlias( PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME ) );
                //strBuilder.AppendFormat( ", {0}",
                //    SqlCommandHelper.EscapeTableColumnNameWithAlias( PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.SSCC.NAME, SelectItemToPrepareDataSet.Columns.SSCC.Name ) );

                qryBuilder.AppendFormat("\r\n FROM  {0}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

                if (filterExpiredProducts)
                {
                    qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
                      SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
                    qryBuilder.AppendFormat(" ON {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME));

                    qryBuilder.AppendFormat("\r\n INNER JOIN {0} {1}",
                      SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                      SqlCommandHelper.EscapeTableName(po_oitm));
                    qryBuilder.Append(qryBuilder.WITH_NOLOCK);
                    qryBuilder.AppendFormat(" ON {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(po_oitm, PmxItemTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME));
                }

                qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                qryBuilder.Append(qryBuilder.WITH_NOLOCK);
                qryBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME));

                qryBuilder.Append(" LEFT JOIN ");
                qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
                qryBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
                qryBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
                qryBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
                qryBuilder.Append(" <> -1 ");




                qryBuilder.AppendFormat("\r\n LEFT JOIN (  {0}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                {
                    qryBuilder.AppendFormat("\r\n    LEFT JOIN {0}",
                        SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                    qryBuilder.AppendFormat(" ON {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME));
                }
                qryBuilder.AppendFormat("\r\n ) ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME));
                qryBuilder.AppendFormat(" AND {0} = {1} ",
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(inputLocationCode, dbTool.CultureInfo));
                qryBuilder.AppendFormat(" AND ( {0} IS NULL OR {0} NOT IN ( SELECT {1} FROM {2} ) )",
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.LogUnitIdentKey.NAME),
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderPreparedComponentTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdOrderDocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME));

                if (filterExpiredProducts)
                {
                    //AND ISNULL(PMX_ITRI.BestBeforeDate, [owor].DueDate + ISNULL(OITM.U_PMX_SLDR, 0)) 
                    // >= [owor].DueDate + ISNULL(OITM.U_PMX_SLDR, 0)

                    //strBuilder.AppendFormat("\r\n    AND ( {0} IS NULL OR {0} >= " + strBuilder.IFNULL + "( {1}, {2} + " + strBuilder.IFNULL + "( {3}, 0 ) ) )",
                    //   SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME),
                    //   SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.BestBeforeDate.NAME),
                    //   SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME),
                    //   SqlCommandHelper.EscapeTableColumnName(po_oitm, PmxItemTable.Columns.ShelfLifeInDaysReception.NAME));


                    qryBuilder.AppendFormat("\r\n  AND ( {5} IS NULL OR ( CASE WHEN {0} = {1} THEN CURRENT_TIMESTAMP WHEN {0} = {2} THEN " + qryBuilder.IFNULL + "( {3}, {4} ) ELSE {4} END < {5} ) )",
                       SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.BestBeforeDateOption.NAME),
                       qryBuilder.AsString(PmxPickProdBestBeforeDateOptionColumnValues.PickDate.VALUE),
                       qryBuilder.AsString(PmxPickProdBestBeforeDateOptionColumnValues.BestBeforeDateFinishedProductAndShelfLife.VALUE),
                       SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.BestBeforeDate.NAME),
                       SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME),
                       SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));


                }


                if (itemsAreLocked)
                {
                    //Items are locked, so only get locked quantities

                    /*INNER JOIN PMX_INLD
ON PMX_INLD.BaseType = '202'
AND PMX_INLD.BaseEntry = 153
AND PMX_INLD.ItemCode = PMX_INVT.ItemCode
AND ISNULL(PMX_INLD.ItemTransactionalInfoKey, 0) = ISNULL(PMX_INVT.ItemTransactionalInfoKey, 0)
AND PMX_INLD.QualityStatusCode = PMX_INVT.QualityStatusCode
AND PMX_INLD.StorLocCode = PMX_INVT.StorLocCode
AND ISNULL(PMX_INLD.LogUnitIdentKey, 0) = ISNULL(PMX_INVT.LogUnitIdentKey, 0)
*/

                    qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
                        SqlCommandHelper.EscapeTableName(PmxInventoryLockingDetailTable.NAME));


                    qryBuilder.AppendFormat("\r\n ON {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseType.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.ProductionOrder, dbTool.CultureInfo));

                    qryBuilder.AppendFormat("\r\n AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(prodOrderDocEntry, dbTool.CultureInfo));

                    qryBuilder.AppendFormat("\r\n  AND {0} = {1} ",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME));

                    qryBuilder.AppendFormat("\r\n  AND {0} = {1} ",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.QualityStatusCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));

                    qryBuilder.AppendFormat("\r\n  AND {0} = {1} ",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.StorLocCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));

                    qryBuilder.AppendFormat("\r\n  AND {0}({1}, 0) = {0}({2}, 0) ",
                        qryBuilder.IFNULL,
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

                    qryBuilder.AppendFormat("\r\n  AND {0}({1}, 0) = {0}({2}, 0) ",
                        qryBuilder.IFNULL,
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.ItemTransactionalInfoKey.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME));

                }


                qryBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(prodOrderDocEntry, dbTool.CultureInfo));

                qryBuilder.AppendFormat(" AND {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryItem.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, dbTool.CultureInfo));

                qryBuilder.Append("\r\n AND ", qryBuilder.IFNULL, "(");
                SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UseForTimeRegistration.NAME);
                qryBuilder.AppendFormat(", {0} ) = {0} ",
                SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));


                if (filter != null)
                {
                    qryBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(filter.ItemCode, dbTool.CultureInfo));
                    if (filter.ItriKey.HasValue)
                    {
                        qryBuilder.AppendFormat(" AND {0} = {1}",
                            SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME),
                            SqlCommandHelper.FormatAndEscapeColumnValue(filter.ItriKey, dbTool.CultureInfo));
                    }
                    else
                    {
                        qryBuilder.AppendFormat(" AND {0} IS NULL",
                            SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME));
                    }
                    if (filter.Luid.HasValue)
                    {
                        qryBuilder.AppendFormat(" AND {0} = {1}",
                            SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME),
                            SqlCommandHelper.FormatAndEscapeColumnValue(filter.Luid, dbTool.CultureInfo));
                    }
                    else
                    {
                        qryBuilder.AppendFormat(" AND {0} IS NULL",
                            SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));
                    }
                    qryBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(filter.QualityStatus, dbTool.CultureInfo));
                }

                if (hasToBeLinedUpItems.HasValue)
                {
                    qryBuilder.AppendFormat("\r\n AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(hasToBeLinedUpItems.Value ? SboBooleanColumnValues.Y.VALUE : SboBooleanColumnValues.N.VALUE, dbTool.CultureInfo));

                }


                qryBuilder.AppendFormat("\r\n ORDER BY {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));


                // * Get already prepared quantities *
                return qryBuilder.ToString();
            }

            internal static string GetPrepairedQuantities(int prodOrderDocEntry, string inputLocationCode, IngredientFilter filter, bool filterExpiredProducts, DbTool dbTool)
            {
                DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

                qryBuilder.AppendFormat("\r\n\r\n SELECT {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdBatchSeqNr.NAME, SelectItemToPrepareDataSet.Columns.PROD_BATCH_SEQ_NR.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, SelectItemToPrepareDataSet.Columns.LINE_NUM.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, SelectItemToPrepareDataSet.Columns.ITEM_CODE.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME, SelectItemToPrepareDataSet.Columns.QUANTITY.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, SelectItemToPrepareDataSet.Columns.LUID.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SelectItemToPrepareDataSet.Columns.STORAGE_LOCATION.Name));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, SelectItemToPrepareDataSet.Columns.SSCC.Name));
                qryBuilder.AppendFormat(", SUM( {0} ) OVER ( PARTITION BY {1} ) AS {2}",
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME),
                    SqlCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.TOTAL_PREPARED_QTY.Name));

                qryBuilder.AppendFormat(", SUM( {0} ) OVER ( PARTITION BY {1}, {2} ) AS {3}",
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdBatchSeqNr.NAME),
                    SqlCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.TOTAL_PREPARED_QTY_BY_BATCH.Name));

                qryBuilder.AppendFormat(",  {0} / " + qryBuilder.IFNULL + "( {1}, 1) AS {2}",
                     SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
                     SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME),
                     SqlCommandHelper.EscapeColumnName(SelectItemToPrepareDataSet.Columns.QTY_PLANNED_BY_BATCH.Name));

                qryBuilder.AppendFormat("\r\n FROM  {0}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

                qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
                qryBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME));

                qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderPreparedComponentTable.NAME));
                qryBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdOrderDocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME));
                qryBuilder.AppendFormat(" AND {2}({0}, {1}) = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdOrderLineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME),
                    qryBuilder.IFNULL);


                qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                qryBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.LogUnitIdentKey.NAME));
                qryBuilder.AppendFormat(" AND {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME));

                qryBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(prodOrderDocEntry, dbTool.CultureInfo));

                qryBuilder.AppendFormat("\r\n ORDER BY {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME));

                return qryBuilder.ToString();
            }

            internal static string GetThinClientScales(string deviceId, CultureInfo dbCulture)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxScaleDefinitionTable.NAME, PmxScaleDefinitionTable.Columns.ProviderType.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxScaleDefinitionTable.NAME, PmxScaleDefinitionTable.Columns.Settings.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxScaleDefinitionTable.NAME, PmxScaleDefinitionTable.Columns.MaxWeight.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxScaleDefinitionTable.NAME, PmxScaleDefinitionTable.Columns.NrOfDecimals.NAME));
                //strBuilder.AppendFormat( ", {0}",
                //    SqlCommandHelper.EscapeTableColumnName( PmxScaleDefinitionTable.NAME, PmxScaleDefinitionTable.Columns.ScaleServerConnStr.NAME ) );

                strBuilder.AppendFormat("\r\n FROM  {0}",
                    SqlCommandHelper.EscapeTableName(PmxThinClientScaleTable.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxScaleDefinitionTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxScaleDefinitionTable.NAME, PmxScaleDefinitionTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxThinClientScaleTable.NAME, PmxThinClientScaleTable.Columns.ScaleDefCode.NAME));

                strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxThinClientScaleTable.NAME, PmxThinClientScaleTable.Columns.ParentCode.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(deviceId, dbCulture));

                strBuilder.AppendFormat("\r\n ORDER BY {0} DESC",
                    SqlCommandHelper.EscapeTableColumnName(PmxScaleDefinitionTable.NAME, PmxScaleDefinitionTable.Columns.MaxWeight.NAME));

                return strBuilder.ToString();
            }

            internal static string GetNextProductionBatchInfo(int prodOrderDocEntry, string itemCode, DbTool dbTool)
            {
                DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

                const string lowerTolQty = "lowerTolQty";

                qryBuilder.AppendFormat("SELECT {0}",
                     SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.NrOfProdBatches.NAME));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.ProdBatchSize.NAME));
                qryBuilder.AppendFormat(", {0}",
                     SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.InternalKey.NAME));
                qryBuilder.AppendFormat(", {0}",
                     SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.SeqNr.NAME));
                qryBuilder.AppendFormat(", {0}",
                     SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.PlannedQty.NAME));
                qryBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.WeighingStatus.NAME));
                qryBuilder.AppendFormat(", {0} * ( 100.0 - {1} ) / 100.0 AS {2}",
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME),
                    SqlCommandHelper.EscapeColumnName(lowerTolQty));
                qryBuilder.AppendFormat(", {0}",
                     SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME + "_" + PmxInventoryTotalTable.Columns.Quantity.NAME));
                qryBuilder.Append("\r\n FROM (");
                {
                    qryBuilder.AppendFormat("\r\n    SELECT {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProdBatchSize.NAME));
                    qryBuilder.AppendFormat(", {0}",
                         SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.InternalKey.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.SeqNr.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.WeighingStatus.NAME));
                    qryBuilder.AppendFormat(", {0} * " + qryBuilder.IFNULL + "( {1}, {2} ) / {3} AS {4}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.PlannedQty.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProdBatchSize.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
                        SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.PlannedQty.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME));
                    qryBuilder.AppendFormat(", SUM( {0} ) AS {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME),
                        SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME + "_" + PmxInventoryTotalTable.Columns.Quantity.NAME));
                    qryBuilder.AppendFormat(", " + qryBuilder.IFNULL + "({0}, 0) AS {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME),
                        SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME)
                        );


                    qryBuilder.AppendFormat("\r\n    FROM  {0}",
                        SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

                    qryBuilder.AppendFormat("\r\n    INNER JOIN {0}",
                        SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));
                    qryBuilder.AppendFormat(" ON {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME));
                    qryBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, dbTool.CultureInfo));

                    qryBuilder.AppendFormat("\r\n    LEFT JOIN {0}",
                        SqlCommandHelper.EscapeTableName(PmxProductionOrderBatchTable.NAME));
                    qryBuilder.AppendFormat(" ON {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.DocEntry.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME));

                    qryBuilder.AppendFormat("\r\n    LEFT JOIN {0}",
                        SqlCommandHelper.EscapeTableName(PmxProductionOrderPreparedComponentTable.NAME));
                    qryBuilder.AppendFormat(" ON {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdOrderDocEntry.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.DocEntry.NAME));
                    qryBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdBatchSeqNr.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.SeqNr.NAME));
                    qryBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME));

                    qryBuilder.AppendFormat("\r\n    LEFT JOIN {0}",
                      SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                    qryBuilder.AppendFormat(" ON {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.LogUnitIdentKey.NAME));
                    qryBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ItemCode.NAME));

                    qryBuilder.AppendFormat("\r\n    WHERE {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(prodOrderDocEntry, dbTool.CultureInfo));

                    qryBuilder.AppendFormat("\r\n    GROUP BY {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProdBatchSize.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.InternalKey.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.SeqNr.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.PlannedQty.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.WeighingStatus.NAME));
                    qryBuilder.AppendFormat(", {0}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME));

                }
                qryBuilder.Append("\r\n ) t");

                qryBuilder.AppendFormat("\r\n ORDER BY CASE WHEN {0} = {1} AND ( {2} IS NULL OR {2} < {3} * ( 100.0 - {4} ) / 100.0 ) AND {5} >= {6} THEN {7}",
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.WeighingStatus.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(PmxProductionStatusColumnValues.Started.VALUE, dbTool.CultureInfo),
                    SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME + "_" + PmxInventoryTotalTable.Columns.Quantity.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.SeqNr.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.SeqNr.NAME));
                qryBuilder.AppendFormat("\r\n    WHEN {0} = {1} THEN 1000000000 + {0}",
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.WeighingStatus.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(PmxProductionStatusColumnValues.Planned.VALUE, dbTool.CultureInfo),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.SeqNr.NAME));
                qryBuilder.AppendFormat("\r\n    ELSE 2000000000 - {0} END",
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderBatchTable.Columns.SeqNr.NAME));

                return qryBuilder.ToString();
            }

            internal static string GetProductionOrderBatchWhereClause(int prodOrderDocEntry, int prodBatchSeqNr, PmxDbConnection dbConn)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("{0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(prodOrderDocEntry, dbConn.DbCultureInfo));
                strBuilder.AppendFormat(" AND {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderBatchTable.NAME, PmxProductionOrderBatchTable.Columns.SeqNr.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(prodBatchSeqNr, dbConn.DbCultureInfo));

                return strBuilder.ToString();
            }

            internal static string GetProductionOrderPreparedComponentWhereClause(int prodOrderDocEntry, int? prodBatchSeqNr, int luid, string itemCode, int? lineNum, PmxDbConnection dbConn)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("{0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdOrderDocEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(prodOrderDocEntry, dbConn.DbCultureInfo));
                strBuilder.AppendFormat(" AND {0} {1} {2}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdBatchSeqNr.NAME),
                    prodBatchSeqNr.HasValue ? "=" : "IS NULL",
                    !prodBatchSeqNr.HasValue ? "" : SqlCommandHelper.FormatAndEscapeColumnValue(prodBatchSeqNr, dbConn.DbCultureInfo));
                strBuilder.AppendFormat(" AND {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.LogUnitIdentKey.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(luid, dbConn.DbCultureInfo));
                strBuilder.AppendFormat(" AND {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, dbConn.DbCultureInfo));
                if (lineNum.HasValue)
                {
                    strBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdOrderLineNum.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(lineNum.Value, dbConn.DbCultureInfo));
                }

                return strBuilder.ToString();
            }

        }

        #endregion
    }
}