using System;
using System.Collections.Generic;
using Produmex.Foundation.Collections;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Threading;
using Produmex.Foundation;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;

namespace Produmex.Sbo.Logex.WorkflowScripts
{
    [FlowCustomization(nameof(Workflows.MainTouchFlowScript))]
	public class WorkflowScript_MainTouchFlowScriptSFF : WorkflowInstanceMainScriptBase
	{
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Private fields
        private CultureInfo m_defaultCultureInfo;
        private PmxOseCompany m_pmxOseCompany;

        // Input parameters
        // <none>

        // Output parameters
        // <none>

        private bool m_logOnFlow_LogOnCanceled;

        public WorkflowScript_MainTouchFlowScriptSFF( WorkflowInstanceBase parent, WorkflowInstanceFactory factory )
            : base(parent, factory)
        {
            // Initialize required sub-flows
            m_logOnFlow = Create("LogOnScript");
            m_logOnFlow.BindParameters(
                new ReadWriteBinder<CultureInfo>("DefaultCultureInfo", this, "m_defaultCultureInfo"),
                new ReadWriteBinder<PmxOseCompany>("PmxOseCompany", this, "m_pmxOseCompany"),
                new ReadWriteBinder<bool>("LogOnCanceled", this, "m_logOnFlow_LogOnCanceled")
            );


		m_MenuOptions.Add(new MenuOption("MSG_BUTTON_COMPONENT_WEIGHING_PRODUCTION", Create("ComponentWeighingProductionScriptSFF")));
            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_PRODUCTION_RECEIPT", Create("ProductionReceiptScript")));


//            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_PRODUCTION", Create("ProductionScript")));
//            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_PACKING", Create("PackingScript")));
//            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_CONSOLIDATION_PACKING", Create("PackingScript").BindParameters(new ReadOnlyBinder<bool>("ConsolidationPacking", true))));
//			m_MenuOptions.Add(new MenuOption("MSG_BUTTON_ITEM_PACKING", Create("PackingScript").BindParameters(new ReadOnlyBinder<bool>("ItemPacking", true))));
//			m_MenuOptions.Add(new MenuOption("MSG_BUTTON_CASH_REGISTER_PACKING", Create("PackingScript").BindParameters(new ReadOnlyBinder<bool>("CashRegisterPacking", true))));

		//m_MenuOptions.Add(new MenuOption("Production script", Create("ProductionScript")));
//            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_WEIGHING", Create("WeighingScript")));
//            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_DISASSEMBLY", Create("DisassemblyScript")));
//            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_DISASSEMBLY_WEIGHT", Create("DisassemblyScript").BindParameters(new ReadOnlyBinder<bool>("UseItemQuantityCombinedScreen", true))));
//            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_REDO_SALES_DELIVERY_CREATED_HOOK", Create("RedoSalesDeliveryCreatedHookScript")));
//            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_REDO_SALES_INVOICE_CREATED_HOOK", Create("RedoSalesInvoiceCreatedHookScript")));
	

			foreach (MenuOption menuOption in m_MenuOptions)
            {
                menuOption.Flow.BindParameters(
                    new ReadOnlyBinder<CultureInfo>("DefaultCultureInfo", this, "m_defaultCultureInfo"),
                    new ReadOnlyBinder<PmxOseCompany>("PmxOseCompany", this, "m_pmxOseCompany")
                );
            }
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Store menu options indexed by button key
            PmxDictionary<string, MenuOption> menuOptionsByButtonKey = new PmxDictionary<string, MenuOption>();
            foreach ( MenuOption menuOption in m_MenuOptions)
            {
                menuOptionsByButtonKey[menuOption.ButtonKey] = menuOption;
            }

            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;

            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;


			while ( true )
            {
            Step_Logon:

                // Run sub-flow LogOn
                base.StartSubFlowAndWait(m_logOnFlow, true);


                if ( m_logOnFlow_LogOnCanceled ) break;

                string initialErrorKey = null;

                int topMenuOptionIndex = 0;

				Step_SelectFunction:

                string[] buttonKeys = new string[ 6 ];
                int btnIdx = 0;
                for ( int i = topMenuOptionIndex; i < m_MenuOptions.Count; i++ )
                {
                    buttonKeys[ btnIdx ] = m_MenuOptions[ i ].ButtonKey;

                    if ( ++btnIdx >= 4 ) break;
                }

                ToolBarButtons toolBarButtons = ToolBarButtons.Stop;
                if ( topMenuOptionIndex > 0 ) toolBarButtons |= ToolBarButtons.Back;
                if ( topMenuOptionIndex + 4 < m_MenuOptions.Count ) toolBarButtons |= ToolBarButtons.Forward;

                session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                    m_defaultCultureInfo, BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_FUNCTION" + ( topMenuOptionIndex / 4 + 1 ).ToString(),
                    "AllowedButtons", buttonKeys,
                    "ScreenType", ScreenType.Menu,
                    "EnabledToolBarButtons", toolBarButtons
                    ),
                    WorkflowId,
                    nameof(MainTouchFlowScript_Screens.SelectButtonScreen1));
                Message msg = WaitForMessage();
                initialErrorKey = null;
                if ( msg.Name.EndsWith(".ButtonClicked") )
                {
                    string buttonKey = ExtractParameter<string>(msg.Parameters, "buttonKey");
                    MenuOption menuOption;
                    if ( menuOptionsByButtonKey.TryGetValue(buttonKey, out menuOption) )
                    {
                        try
                        {
                            base.StartSubFlowAndWait(menuOption.Flow, true);
                        }
                        catch ( Exception ex )
                        {

                            s_log.Error(String.Format("Exception in workflow behind button {0}", menuOption.ButtonKey), ex);

                            ProdumexError pmxError;
                            ex = ExtractExceptionAndPmxErrorForDisplay(ex, out pmxError);

                            session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                m_defaultCultureInfo, BuildParamCollection(
                                "MessageKey", ExceptionHelper.Format( ex, m_defaultCultureInfo ),
                                "NoTranslationOfMessageKey", true,
                                "PmxError", pmxError,
                                // "MessageAddendum", ReflectionHelper.CompactStackTrace( ex.StackTrace ),
                                "ShowButton", true
                                ),
                                WorkflowId,
                                nameof(MainTouchFlowScript_Screens.ShowMessageScreen1));
                            msg = WaitForMessage();
                        }

                        goto Step_SelectFunction;
                    }
                }
                if ( msg.Name.EndsWith( ".BackRequested" ) )
                {
                    topMenuOptionIndex = Math.Max( 0, topMenuOptionIndex - 4 );
                    goto Step_SelectFunction;
                }
                if ( msg.Name.EndsWith( ".ForwardRequested" ) )
                {
                    topMenuOptionIndex += 4;
                    goto Step_SelectFunction;
                }
                if ( msg.Name.EndsWith( ".StopRequested" ) )
                {
                    sboProviderService.LogOffWithSessionId( session.Id );
                    goto Step_Logon;
                }

                initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                goto Step_SelectFunction;
            }
        }

        #endregion
    }
}

