using System;
using System.Globalization;
using System.Reflection;
using System.Data;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Wwf.Sbo.LocalServices;

namespace Produmex.Sbo.Logex.WorkflowScripts
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_AfterLogisticUnitItemEnteredHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;       
        public ReadOnlyBinder<int?> ContainerDocEntry;
        public ReadWriteBinder<LogisticUnitItemGoodReceipt> LogUnitItem;

        // Output parameters * do not change *

        public ReadWriteBinder<bool> BackRequested;
        // Sub-flows
        // <none>

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_AfterLogisticUnitItemEnteredHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_AfterLogisticUnitItemEnteredHookScript( WorkflowInstanceBase parent, WorkflowInstanceFactory factory )
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
		Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
		Message msg = null;
		string initialErrorKey = null;
		string item = "";
		string FreeText = "";
		string query = @"SELECT U_PMX_WCRE FROM OITM WHERE ItemCode = '" + LogUnitItem.Get().ItemCode + "'";
 		double weigth = 0;
            DataSet dsCustomQuery = null;
		DataSet dsItems = null;
		DataSet dsItemsWeight = null;

            dsCustomQuery = sboProviderService.RunView(false, null, null, query);
            if (dsCustomQuery.Tables.Count > 0)
            {
			item = dsCustomQuery.Tables[0].Rows[0]["U_PMX_WCRE"].ToString();
			s_log.Error("ROPI: weight managed item: |" + item + "|"); 
            	if ( item == "Y" )
			{

				query = "SELECT " + 
					"	DISTINCT PMX_OITM_MANAGED_BY_PMX.ItemCode AS ProductCode, " + 
					"	PMX_OITM_MANAGED_BY_PMX.U_PMX_CUDE AS ProductDescription, " + 
					"	PMX_OITM_MANAGED_BY_PMX.CodeBars AS GTIN," + 
					"	PMX_OITM_MANAGED_BY_PMX.U_PMX_HBBD, " + 
					"	PMX_OITM_MANAGED_BY_PMX.U_PMX_PILR, " + 
					"	PMX_OITM_MANAGED_BY_PMX.ManBtchNum, " + 
					"	PMX_OITM_MANAGED_BY_PMX.U_PMX_LOUN, " + 
					"	CAST(SWeight1/1000 as float ) as NumInBuy, " + 
					"	PMX_OITM_MANAGED_BY_PMX.BuyUnitMsr, " + 
					"	'kg' as InvntryUom, " + 
					"	PMX_OITM_MANAGED_BY_PMX.CodeBars AS CodeBars, " + 
					"	PMX_OITM_MANAGED_BY_PMX.ItemName " + 
					"FROM " + 
					"PMX_OITM_MANAGED_BY_PMX WITH (NOLOCK)  " + 
					"WHERE  U_PMX_LOCA = 'Y'" + 
					"ORDER BY ProductDescription	";

				dsItems = sboProviderService.RunView(false, null, null, query);
				
				msg = null;
				session.ShowCustomizedScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
      				"InitialErrorKey", initialErrorKey,
	            		"TitleKey", "Select Logistic Carrier",
      	      		"ProductDS", dsItems
            			),
				WorkflowId,
      			nameof(ChecksScript_Screens.SelectProductScreen1));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".ProductSelected"))
				{
					query = @"SELECT CAST(SWeight1/1000 as float ) as Weight FROM OITM WHERE ItemCode = '" + ExtractParameter<string>( msg.Parameters, "itemCode" ) + "'";
					dsItemsWeight = sboProviderService.RunView(false, null, null, query);
					if (dsItemsWeight.Tables.Count > 0)
					{
						weigth = Double.Parse(dsItemsWeight.Tables[0].Rows[0]["Weight"].ToString());
					}
				}
				FreeText = weigth.ToString();
				double ItemWeight = LogUnitItem.Get().Quantity - weigth;
				LogUnitItem.Get().Quantity = ItemWeight;
/*
				s_log.Error("ROPI: weight managed, LC: |" + FreeText + "|");
 				msg = null;

		            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
            		this.DefaultCultureInfo, BuildParamCollection(
		            	"MessageKey", FreeText  + "Calculation: Pallet 15 kg, SumWeight: " + LogUnitItem.Get().Quantity.ToString() + ", ItemWeight = " + ItemWeight.ToString(),
            		      "ShowButton", true
	                  ));
				msg = WaitForMessage();
*/
			}
		}
        }

        #endregion
    }
}
